/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * SetWorldTimeAction.cpp
 *
 *  Created on: May 8, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"
#include "World.hpp"
#include "WorldTime.hpp"

#include <iostream>
#include <string>

#include "Actions/WorldAction/SetWorldTimeAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "SetWorldTimeAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
Action::state_ptr WorldSetWorldTimeAction::performCall() {
  // create undo state
  int oldtime;
  oldtime = WorldTime::getTime();
  WorldSetWorldTimeState *UndoState = new WorldSetWorldTimeState(oldtime, params);

  World::getInstance().setTime(params.newtime.get());
  LOG(0, "Current time step is now: " << WorldTime::getTime() << ".");
  return Action::state_ptr(UndoState);
}

Action::state_ptr WorldSetWorldTimeAction::performUndo(Action::state_ptr _state) {
  WorldSetWorldTimeState *state = assert_cast<WorldSetWorldTimeState*>(_state.get());

  World::getInstance().setTime(state->oldtime);
  LOG(0, "Current time step is now again: " << WorldTime::getTime() << ".");

  return Action::state_ptr(_state);
}

Action::state_ptr WorldSetWorldTimeAction::performRedo(Action::state_ptr _state){
  WorldSetWorldTimeState *state = assert_cast<WorldSetWorldTimeState*>(_state.get());

  World::getInstance().setTime(state->params.newtime.get());
  LOG(0, "Current time step is now: " << WorldTime::getTime() << ".");

  return Action::state_ptr(_state);
}

bool WorldSetWorldTimeAction::canUndo() {
  return true;
}

bool WorldSetWorldTimeAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
